@@ -564,6 +564,23 @@ public class COSDictionary extends COSBase
         return getCOSName(key, null);
     }
 
+    /**
+     * This is a convenience method that will get the dictionary object that is expected to be a COSArray. Null is
+     * returned if the entry does not exist in the dictionary.
+     *
+     * @param key The key to the item in the dictionary.
+     * @return The COSArray.
+     */
+    public COSArray getCOSArray(COSName key)
+    {
+        COSBase array = getDictionaryObject(key);
+        if (array instanceof COSArray)
+        {
+            return (COSArray) array;
+        }
+        return null;
+    }
+
     /**
      * Convenience method that will get the dictionary object that is expected to be a name. Default is returned if the
      * entry does not exist in the dictionary.
