@@ -34,6 +34,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     // A
     public static final COSName A = newCommonInstance("A");
     public static final COSName AA = newCommonInstance("AA");
+    public static final COSName ABSOLUTE_COLORIMETRIC = newCommonInstance("AbsoluteColorimetric");
     public static final COSName AC = newCommonInstance("AC");
     public static final COSName ACRO_FORM = newCommonInstance("AcroForm");
     public static final COSName ACTUAL_TEXT = newCommonInstance("ActualText");
@@ -46,12 +47,16 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName AF = newCommonInstance("AF");
     public static final COSName AFTER = newCommonInstance("After");
     public static final COSName AIS = newCommonInstance("AIS");
+    public static final COSName ALL_OFF = newCommonInstance("AllOff");
+    public static final COSName ALL_ON = newCommonInstance("AllOn");
     public static final COSName ALT = newCommonInstance("Alt");
     public static final COSName ALPHA = newCommonInstance("Alpha");
     public static final COSName ALTERNATE = newCommonInstance("Alternate");
     public static final COSName ANNOT = newCommonInstance("Annot");
     public static final COSName ANNOTS = newCommonInstance("Annots");
     public static final COSName ANTI_ALIAS = newCommonInstance("AntiAlias");
+    public static final COSName ANY_OFF = newCommonInstance("AnyOff");
+    public static final COSName ANY_ON = newCommonInstance("AnyOn");
     public static final COSName AP = newCommonInstance("AP");
     public static final COSName AP_REF = newCommonInstance("APRef");
     public static final COSName APP = newCommonInstance("App");
@@ -125,7 +130,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName CMAPNAME = newCommonInstance("CMapName");
     public static final COSName CMYK = newCommonInstance("CMYK");
     public static final COSName CO = newCommonInstance("CO");
-    public static final COSName COLOR = new COSName("Color");
+    public static final COSName COLOR = newCommonInstance("Color");
     public static final COSName COLOR_BURN = newCommonInstance("ColorBurn");
     public static final COSName COLOR_DODGE = newCommonInstance("ColorDodge");
     public static final COSName COLORANTS = newCommonInstance("Colorants");
@@ -187,7 +192,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName DOC_OPEN = newCommonInstance("DocOpen");
     public static final COSName DOC_TIME_STAMP = newCommonInstance("DocTimeStamp");
     public static final COSName DOCMDP = newCommonInstance("DocMDP");
-    public static final COSName DOCUMENT = new COSName("Document");
+    public static final COSName DOCUMENT = newCommonInstance("Document");
     public static final COSName DOMAIN = newCommonInstance("Domain");
     public static final COSName DOS = newCommonInstance("DOS");
     public static final COSName DP = newCommonInstance("DP");
@@ -269,7 +274,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName HIDE_MENUBAR = newCommonInstance("HideMenubar");
     public static final COSName HIDE_TOOLBAR = newCommonInstance("HideToolbar");
     public static final COSName HIDE_WINDOWUI = newCommonInstance("HideWindowUI");
-    public static final COSName HUE = new COSName("Hue");
+    public static final COSName HUE = newCommonInstance("Hue");
     // I
     public static final COSName I = newCommonInstance("I");
     public static final COSName IC = newCommonInstance("IC");
@@ -287,6 +292,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName INDEXED = newCommonInstance("Indexed");
     public static final COSName INFO = newCommonInstance("Info");
     public static final COSName INKLIST = newCommonInstance("InkList");
+    public static final COSName INTENT = newCommonInstance("Intent");
     public static final COSName INTERPOLATE = newCommonInstance("Interpolate");
     public static final COSName IT = newCommonInstance("IT");
     public static final COSName ITALIC_ANGLE = newCommonInstance("ItalicAngle");
@@ -343,7 +349,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName MCID = newCommonInstance("MCID");
     public static final COSName MDP = newCommonInstance("MDP");
     public static final COSName MEDIA_BOX = newCommonInstance("MediaBox");
-    public static final COSName MEASURE = new COSName("Measure");
+    public static final COSName MEASURE = newCommonInstance("Measure");
     public static final COSName METADATA = newCommonInstance("Metadata");
     public static final COSName MISSING_WIDTH = newCommonInstance("MissingWidth");
     public static final COSName MK = newCommonInstance("MK");
@@ -356,7 +362,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName NAME = newCommonInstance("Name");
     public static final COSName NAMES = newCommonInstance("Names");
     public static final COSName NEED_APPEARANCES = newCommonInstance("NeedAppearances");
-    public static final COSName NEW_WINDOW = new COSName("NewWindow");
+    public static final COSName NEW_WINDOW = newCommonInstance("NewWindow");
     public static final COSName NEXT = newCommonInstance("Next");
     public static final COSName NM = newCommonInstance("NM");
     public static final COSName NON_EFONT_NO_WARN = newCommonInstance("NonEFontNoWarn");
@@ -372,6 +378,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName OC = newCommonInstance("OC");
     public static final COSName OCG = newCommonInstance("OCG");
     public static final COSName OCGS = newCommonInstance("OCGs");
+    public static final COSName OCMD = newCommonInstance("OCMD");
     public static final COSName OCPROPERTIES = newCommonInstance("OCProperties");
     public static final COSName OE = newCommonInstance("OE");
     public static final COSName OFF = newCommonInstance("OFF");
@@ -406,10 +413,12 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName PARENT = newCommonInstance("Parent");
     public static final COSName PARENT_TREE = newCommonInstance("ParentTree");
     public static final COSName PARENT_TREE_NEXT_KEY = newCommonInstance("ParentTreeNextKey");
+    public static final COSName PATH = newCommonInstance("Path");
     public static final COSName PATTERN = newCommonInstance("Pattern");
     public static final COSName PATTERN_TYPE = newCommonInstance("PatternType");
     public static final COSName PDF_DOC_ENCODING = newCommonInstance("PDFDocEncoding");
     public static final COSName PERMS = newCommonInstance("Perms");
+    public static final COSName PERCEPTUAL = newCommonInstance("Perceptual");
     public static final COSName PG = newCommonInstance("Pg");
     public static final COSName PIECE_INFO = newCommonInstance("PieceInfo");
     public static final COSName PMD = newCommonInstance("PMD");
@@ -437,6 +446,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName RD = newCommonInstance("RD");
     public static final COSName REASON = newCommonInstance("Reason");
     public static final COSName REASONS = newCommonInstance("Reasons");
+    public static final COSName RELATIVE_COLORIMETRIC = newCommonInstance("RelativeColorimetric");
     public static final COSName RECIPIENTS = newCommonInstance("Recipients");
     public static final COSName RECT = newCommonInstance("Rect");
     public static final COSName REGISTRY = newCommonInstance("Registry");
@@ -455,7 +465,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     // S
     public static final COSName S = newCommonInstance("S");
     public static final COSName SA = newCommonInstance("SA");
-    public static final COSName SATURATION = new COSName("Saturation");
+    public static final COSName SATURATION = newCommonInstance("Saturation");
     public static final COSName SCREEN = newCommonInstance("Screen");
     public static final COSName SE = newCommonInstance("SE");
     public static final COSName SEPARATION = newCommonInstance("Separation");
@@ -529,9 +539,12 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName UNIX = newCommonInstance("Unix");
     public static final COSName URI = newCommonInstance("URI");
     public static final COSName URL = newCommonInstance("URL");
+    public static final COSName URL_TYPE = newCommonInstance("URLType");
+    public static final COSName USER_UNIT = newCommonInstance("UserUnit");
     // V
     public static final COSName V = newCommonInstance("V");
     public static final COSName V2 = newCommonInstance("V2");
+    public static final COSName VE = newCommonInstance("VE");
     public static final COSName VERISIGN_PPKVS = newCommonInstance("VeriSign.PPKVS");
     public static final COSName VERSION = newCommonInstance("Version");
     public static final COSName VERTICES = newCommonInstance("Vertices");
@@ -539,8 +552,8 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName VIEW_AREA = newCommonInstance("ViewArea");
     public static final COSName VIEW_CLIP = newCommonInstance("ViewClip");
     public static final COSName VIEWER_PREFERENCES = newCommonInstance("ViewerPreferences");
-    public static final COSName VOLUME = new COSName("Volume");
-    public static final COSName VP = new COSName("VP");
+    public static final COSName VOLUME = newCommonInstance("Volume");
+    public static final COSName VP = newCommonInstance("VP");
     // W
     public static final COSName W = newCommonInstance("W");
     public static final COSName W2 = newCommonInstance("W2");
