@@ -17,11 +17,11 @@
 package org.sejda.sambox.pdmodel;
 
 import static java.util.Optional.ofNullable;
+import static org.sejda.commons.util.RequireUtils.requireNotBlank;
 import static org.sejda.io.CountingWritableByteChannel.from;
 import static org.sejda.sambox.cos.DirectCOSObject.asDirectObject;
 import static org.sejda.sambox.util.SpecVersionUtils.V1_4;
 import static org.sejda.sambox.util.SpecVersionUtils.isAtLeast;
-import static org.sejda.util.RequireUtils.requireNotBlank;
 
 import java.awt.Point;
 import java.awt.image.DataBuffer;
@@ -41,8 +41,10 @@ import java.util.Optional;
 import java.util.Set;
 
 import org.apache.fontbox.ttf.TrueTypeFont;
+import org.sejda.commons.util.IOUtils;
 import org.sejda.io.CountingWritableByteChannel;
 import org.sejda.io.SeekableSources;
+import org.sejda.sambox.SAMBox;
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSBase;
 import org.sejda.sambox.cos.COSDictionary;
@@ -64,14 +66,12 @@ import org.sejda.sambox.pdmodel.encryption.PDEncryption;
 import org.sejda.sambox.pdmodel.encryption.SecurityHandler;
 import org.sejda.sambox.pdmodel.font.Subsettable;
 import org.sejda.sambox.pdmodel.graphics.color.PDDeviceRGB;
-import org.sejda.sambox.util.Version;
-import org.sejda.util.IOUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
  * This is the in-memory representation of the PDF document.
- * 
+ *
  * @author Ben Litchfield
  */
 public class PDDocument implements Closeable
@@ -109,6 +109,7 @@ public class PDDocument implements Closeable
     private SecurityHandler securityHandler;
     private boolean open = true;
     private OnClose onClose = () -> LOG.debug("Closing document");
+    private OnBeforeWrite onBeforeWrite = () -> LOG.trace("About to write document");
     private ResourceCache resourceCache = new DefaultResourceCache();
 
     // fonts to subset before saving
@@ -127,7 +128,7 @@ public class PDDocument implements Closeable
 
     /**
      * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
-     * 
+     *
      * @param document The COSDocument that this document wraps.
      */
     public PDDocument(COSDocument document)
@@ -137,7 +138,7 @@ public class PDDocument implements Closeable
 
     /**
      * Constructor that uses an existing document. The COSDocument that is passed in must be valid.
-     * 
+     *
      * @param document The COSDocument that this document wraps.
      * @param securityHandler
      */
@@ -150,7 +151,7 @@ public class PDDocument implements Closeable
     /**
      * This will add a page to the document. This is a convenience method, that will add the page to the root of the
      * hierarchy and set the parent of the page to the root.
-     * 
+     *
      * @param page The page to add to the document.
      */
     public void addPage(PDPage page)
@@ -161,7 +162,7 @@ public class PDDocument implements Closeable
 
     /**
      * Remove the page from the document.
-     * 
+     *
      * @param page The page to remove from the document.
      */
     public void removePage(PDPage page)
@@ -172,7 +173,7 @@ public class PDDocument implements Closeable
 
     /**
      * Remove the page from the document.
-     * 
+     *
      * @param pageNumber 0 based index to page number.
      */
     public void removePage(int pageNumber)
@@ -186,10 +187,10 @@ public class PDDocument implements Closeable
      * file. The scratch file is associated with the document. If you are adding a page to this document from another
      * document and want to copy the contents to this document's scratch file then use this method otherwise just use
      * the addPage method.
-     * 
+     *
      * @param page The page to import.
      * @return The page that was imported.
-     * 
+     *
      */
     public PDPage importPage(PDPage page)
     {
@@ -224,7 +225,7 @@ public class PDDocument implements Closeable
 
     /**
      * This will get the document info dictionary. This is guaranteed to not return null.
-     * 
+     *
      * @return The documents /Info dictionary
      */
     public PDDocumentInformation getDocumentInformation()
@@ -241,8 +242,8 @@ public class PDDocument implements Closeable
 
     /**
      * This will set the document information for this document.
-     * 
-     * @param info The updated document information.
+     *
+     * @param documentInformation The updated document information.
      */
     public void setDocumentInformation(PDDocumentInformation documentInformation)
     {
@@ -253,7 +254,7 @@ public class PDDocument implements Closeable
 
     /**
      * This will get the document CATALOG. This is guaranteed to not return null.
-     * 
+     *
      * @return The documents /Root dictionary
      */
     public PDDocumentCatalog getDocumentCatalog()
@@ -334,7 +335,7 @@ public class PDDocument implements Closeable
      * method returns the access permission for a document owner (ie can do everything). The returned object is in read
      * only mode so that permissions cannot be changed. Methods providing access to content should rely on this object
      * to verify if the current user is allowed to proceed.
-     * 
+     *
      * @return the access permissions for the current user on the document.
      */
     public AccessPermission getCurrentAccessPermission()
@@ -368,7 +369,7 @@ public class PDDocument implements Closeable
      * not allowed.
      *
      * @param newVersion the new PDF version
-     * 
+     *
      */
     public void setVersion(String newVersion)
     {
@@ -392,7 +393,7 @@ public class PDDocument implements Closeable
     /**
      * If the document is not at the given version or above, it sets the version of the PDF specification to which the
      * document conforms.
-     * 
+     *
      * @param version
      */
     public void requireMinVersion(String version)
@@ -406,7 +407,7 @@ public class PDDocument implements Closeable
 
     /**
      * Sets an action to be performed right before this {@link PDDocument} is closed.
-     * 
+     *
      * @param onClose
      */
     public void setOnCloseAction(OnClose onClose)
@@ -414,6 +415,12 @@ public class PDDocument implements Closeable
         requireOpen();
         this.onClose = onClose.andThen(this.onClose);
     }
+    
+    public void setOnBeforeWriteAction(OnBeforeWrite onBeforeWrite)
+    {
+        requireOpen();
+        this.onBeforeWrite = onBeforeWrite.andThen(this.onBeforeWrite);
+    }
 
     private void requireOpen() throws IllegalStateException
     {
@@ -426,7 +433,7 @@ public class PDDocument implements Closeable
     /**
      * Generates file identifier as defined in the chap 14.4 PDF 32000-1:2008 and sets it as first and second value for
      * the ID array in the document trailer.
-     * 
+     *
      * @param md5Update
      * @param encContext
      */
@@ -440,7 +447,7 @@ public class PDDocument implements Closeable
     }
 
     /**
-     * 
+     *
      * @param md5Update
      * @return a newly generated ID based on the input bytes, current timestamp and some other information, to be used
      * as value of the ID array in the document trailer.
@@ -465,7 +472,7 @@ public class PDDocument implements Closeable
 
     /**
      * Writes the document to the given {@link File}. The document is closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param file
      * @param options
@@ -478,7 +485,7 @@ public class PDDocument implements Closeable
 
     /**
      * Writes the document to the file corresponding the given file name. The document is closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param filename
      * @param options
@@ -491,7 +498,7 @@ public class PDDocument implements Closeable
 
     /**
      * Writes the document to the given {@link WritableByteChannel}. The document is closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param channel
      * @param options
@@ -504,7 +511,7 @@ public class PDDocument implements Closeable
 
     /**
      * Writes the document to the given {@link OutputStream}. The document is closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param out
      * @param options
@@ -518,7 +525,7 @@ public class PDDocument implements Closeable
     /**
      * Writes the document to the given {@link File} encrypting it using the given security. The document is closed once
      * written.
-     * 
+     *
      * @see PDDocument#close()
      * @param file
      * @param security
@@ -534,7 +541,7 @@ public class PDDocument implements Closeable
     /**
      * Writes the document to the file corresponding the given file name encrypting it using the given security. The
      * document is closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param filename
      * @param security
@@ -550,7 +557,7 @@ public class PDDocument implements Closeable
     /**
      * Writes the document to the given {@link WritableByteChannel} encrypting it using the given security. The document
      * is closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param channel
      * @param security
@@ -566,7 +573,7 @@ public class PDDocument implements Closeable
     /**
      * Writes the document to the given {@link OutputStream} encrypting it using the given security. The document is
      * closed once written.
-     * 
+     *
      * @see PDDocument#close()
      * @param out
      * @param security
@@ -583,11 +590,16 @@ public class PDDocument implements Closeable
             WriteOption... options) throws IOException
     {
         requireOpen();
-        getDocumentInformation().setProducer("SAMBox " + Version.getVersion() + " (www.sejda.org)");
+
+        getDocumentInformation().setProducer(SAMBox.PRODUCER);
         getDocumentInformation().setModificationDate(Calendar.getInstance());
         for (Subsettable font : fontsToSubset)
         {
-            font.subset();
+            try {
+                font.subset();
+            } catch (Exception e) {
+                LOG.warn("Exception occurred while subsetting font: " + font, e);
+            }
         }
         fontsToSubset.clear();
         Optional<EncryptionContext> encryptionContext = ofNullable(
@@ -596,6 +608,7 @@ public class PDDocument implements Closeable
                 encryptionContext);
         try (PDDocumentWriter writer = new PDDocumentWriter(output, encryptionContext, options))
         {
+            onBeforeWrite.onBeforeWrite();
             writer.write(this);
         }
         finally
@@ -613,22 +626,25 @@ public class PDDocument implements Closeable
     }
 
     /**
-     * Closes the {@link PDDocument} executing an onClose action is set. Once closed the document is pretty much
-     * unusable since most of the mothods requires an open document.
-     * 
+     * Closes the {@link PDDocument} executing the set onClose action. Once closed the document is pretty much unusable
+     * since most of the methods requires an open document.
+     *
      * @see PDDocument#setOnCloseAction(OnClose)
      */
     @Override
     public void close() throws IOException
     {
-        onClose.onClose();
-        this.resourceCache.clear();
-        this.open = false;
+        if (isOpen())
+        {
+            onClose.onClose();
+            this.resourceCache.clear();
+            this.open = false;
+        }
     }
 
     /**
      * Action to be performed before the {@link PDDocument} is close
-     * 
+     *
      * @author Andrea Vacondio
      */
     @FunctionalInterface
@@ -636,8 +652,6 @@ public class PDDocument implements Closeable
     {
         /**
          * Sets an action to be performed right before this {@link PDDocument} is closed.
-         * 
-         * @param onClose
          */
         void onClose() throws IOException;
 
@@ -651,6 +665,24 @@ public class PDDocument implements Closeable
         }
     }
 
+    /**
+     * Action to be performed right before this {@link PDDocument} is written to a file.
+     */
+    @FunctionalInterface
+    public static interface OnBeforeWrite
+    {
+        void onBeforeWrite() throws IOException;
+
+        default OnBeforeWrite andThen(OnBeforeWrite after)
+        {
+            Objects.requireNonNull(after);
+            return () -> {
+                onBeforeWrite();
+                after.onBeforeWrite();
+            };
+        }
+    }
+
     /**
      * Returns the resource cache associated with this document, or null if there is none.
      */
