@@ -18,6 +18,14 @@ package org.sejda.sambox.pdmodel;
 
 import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
+import static java.util.stream.Collectors.toList;
+import static org.sejda.sambox.cos.COSName.DECODE_PARMS;
+import static org.sejda.sambox.cos.COSName.DL;
+import static org.sejda.sambox.cos.COSName.F;
+import static org.sejda.sambox.cos.COSName.FILTER;
+import static org.sejda.sambox.cos.COSName.F_DECODE_PARMS;
+import static org.sejda.sambox.cos.COSName.F_FILTER;
+import static org.sejda.sambox.cos.COSName.LENGTH;
 
 import java.awt.Point;
 import java.awt.geom.Point2D;
@@ -26,6 +34,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.SequenceInputStream;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
@@ -61,11 +70,14 @@ import org.slf4j.LoggerFactory;
 public class PDPage implements COSObjectable, PDContentStream
 {
     private static final Logger LOG = LoggerFactory.getLogger(PDPage.class);
+    private static final List<COSName> VALID_STREAM_KEYS = Arrays.asList(LENGTH, FILTER,
+            DECODE_PARMS, F, F_FILTER, F_DECODE_PARMS, DL);
 
     private final COSDictionary page;
     private PDResources pageResources;
     private ResourceCache resourceCache;
     private PDRectangle mediaBox;
+    private COSDictionary pageTreeParent;
 
     /**
      * Creates a new PDPage instance for embedding, with a size of U.S. Letter (8.5 x 11 inches).
@@ -108,6 +120,18 @@ public class PDPage implements COSObjectable, PDContentStream
         this.resourceCache = resourceCache;
     }
 
+    /**
+     * Creates a new instance of PDPage for reading.
+     *
+     * @param pageDictionary A page dictionary in a PDF document.
+     */
+    PDPage(COSDictionary pageDictionary, ResourceCache resourceCache, COSDictionary pageTreeParent)
+    {
+        page = pageDictionary;
+        this.resourceCache = resourceCache;
+        this.pageTreeParent = pageTreeParent;
+    }
+
     /**
      * Convert this standard java object to a COS object.
      * 
@@ -137,10 +161,18 @@ public class PDPage implements COSObjectable, PDContentStream
             COSArray array = (COSArray) base;
             for (int i = 0; i < array.size(); i++)
             {
-                COSStream stream = (COSStream) array.getObject(i);
-                if (nonNull(stream))
+                COSBase baseObject = array.getObject(i);
+                if(baseObject instanceof COSStream) 
                 {
-                    streams.add(new PDStream(stream));
+                    COSStream stream = (COSStream) baseObject;
+                    if (nonNull(stream))
+                    {
+                        streams.add(new PDStream(stream));
+                    }    
+                }
+                else
+                {
+                    LOG.warn("Page has contents object that is not a stream: " + baseObject);
                 }
             }
         }
@@ -201,7 +233,7 @@ public class PDPage implements COSObjectable, PDContentStream
         if (pageResources == null)
         {
             pageResources = new PDResources(ofNullable(
-                    (COSDictionary) PDPageTree.getInheritableAttribute(page, COSName.RESOURCES))
+                    (COSDictionary) PDPageTree.getInheritableAttribute(page, COSName.RESOURCES, COSDictionary.class))
                             .orElseGet(() -> {
                                 COSDictionary emptyRes = new COSDictionary();
                                 // it's illegal for a page to not have resources, either direct or inherited. According
@@ -834,7 +866,7 @@ public class PDPage implements COSObjectable, PDContentStream
             return null;
         }
         COSArray array = (COSArray) base;
-        List<PDViewportDictionary> viewports = new ArrayList<PDViewportDictionary>();
+        List<PDViewportDictionary> viewports = new ArrayList<>();
         for (int i = 0; i < array.size(); ++i)
         {
             COSBase base2 = array.getObject(i);
@@ -869,4 +901,48 @@ public class PDPage implements COSObjectable, PDContentStream
         }
         page.setItem(COSName.VP, array);
     }
+
+    /**
+     * @return the user unit. This is a positive number that shall give the size of default user space units, in
+     * multiples of 1/72 inch, or 1 if it hasn't been set. This is supported by PDF 1.6 and higher.
+     */
+    public float getUserUnit()
+    {
+        float userUnit = page.getFloat(COSName.USER_UNIT, 1.0f);
+        return userUnit > 0 ? userUnit : 1.0f;
+    }
+
+    /**
+     * Get the user unit. This is a positive number that shall give the size of default user space units, in multiples
+     * of 1/72 inch. This is supported by PDF 1.6 and higher.
+     *
+     * @param userUnit throws IllegalArgumentException if the parameter is not positive.
+     */
+    public void setUserUnit(float userUnit)
+    {
+        if (userUnit <= 0)
+        {
+            throw new IllegalArgumentException("User unit must be positive");
+        }
+        page.setFloat(COSName.USER_UNIT, userUnit);
+    }
+
+    /**
+     * Removes from the page dictionary all non spec compliant keys.
+     * 
+     * @see https://github.com/torakiki/pdfsam/issues/383
+     */
+    public void sanitizeDictionary()
+    {
+        getContentStreams().forEachRemaining(s -> {
+            COSStream stream = s.getCOSObject();
+            List<COSName> invalid = stream.keySet().stream()
+                    .filter(k -> !VALID_STREAM_KEYS.contains(k)).collect(toList());
+            invalid.forEach(stream::removeItem);
+        });
+    }
+
+    public COSDictionary getPageTreeParent() {
+        return pageTreeParent;
+    }
 }
