@@ -20,16 +20,9 @@ import static java.util.Objects.isNull;
 import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
 import static java.util.stream.Collectors.toList;
-import static org.sejda.util.RequireUtils.requireNotNullArg;
+import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 
-import java.util.ArrayDeque;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Objects;
-import java.util.Queue;
-import java.util.Spliterator;
-import java.util.Spliterators;
+import java.util.*;
 import java.util.stream.Stream;
 import java.util.stream.StreamSupport;
 
@@ -103,6 +96,24 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
         this.document = document;
     }
 
+    /**
+     * Similar to {@link #getInheritableAttribute(COSDictionary, COSName)} but also checks the returned element 
+     * matches the expected type class
+     * @param node
+     * @param key
+     * @param clazz expected type class
+     * @param <T>
+     * @return
+     */
+    public static <T extends COSBase> COSBase getInheritableAttribute(COSDictionary node, COSName key, Class<T> clazz) {
+        COSBase result = getInheritableAttribute(node, key);
+        if(clazz.isInstance(result)) {
+            return result;
+        } else {
+            return null;
+        }
+    }
+
     /**
      * Returns the given attribute, inheriting from parent tree nodes if necessary.
      *
@@ -207,6 +218,10 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
         @Override
         public PDPage next()
         {
+            if (!hasNext())
+            {
+                throw new NoSuchElementException();
+            }
             COSDictionary next = queue.poll();
 
             sanitizeType(next);
@@ -252,6 +267,10 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
         @Override
         public COSDictionary next()
         {
+            if (!hasNext())
+            {
+                throw new NoSuchElementException();
+            }
             return queue.poll();
         }
 
@@ -269,12 +288,13 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      */
     public PDPage get(int index)
     {
-        COSDictionary dict = get(index + 1, root, 0);
+        PageAndPageTreeParent res = get(index + 1, root, 0, null);
+        COSDictionary dict = res.node;
 
         sanitizeType(dict);
 
         ResourceCache resourceCache = document != null ? document.getResourceCache() : null;
-        return new PDPage(dict, resourceCache);
+        return new PDPage(dict, resourceCache, res.parent);
     }
 
     private static void sanitizeType(COSDictionary dictionary)
@@ -298,9 +318,10 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      * @param pageNum 1-based page number
      * @param node page tree node to search
      * @param encountered number of pages encountered so far
+     * @param pageTreeParent the parent node, as determined traversing the page tree top -> down
      * @return COS dictionary of the Page object
      */
-    private COSDictionary get(int pageNum, COSDictionary node, int encountered)
+    private PageAndPageTreeParent get(int pageNum, COSDictionary node, int encountered, COSDictionary pageTreeParent)
     {
         if (pageNum < 0)
         {
@@ -324,7 +345,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
                         if (pageNum <= encountered + kidCount)
                         {
                             // it's this kid
-                            return get(pageNum, kid, encountered);
+                            return get(pageNum, kid, encountered, node);
                         }
                         encountered += kidCount;
                     }
@@ -335,7 +356,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
                         if (pageNum == encountered)
                         {
                             // it's this page
-                            return get(pageNum, kid, encountered);
+                            return get(pageNum, kid, encountered, node);
                         }
                     }
                 }
@@ -350,7 +371,7 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
         }
         if (encountered == pageNum)
         {
-            return node;
+            return new PageAndPageTreeParent(node, pageTreeParent);
         }
 
         throw new PageNotFoundException("Unable to find page " + pageNum + " in " + getSourcePath(),
@@ -448,8 +469,8 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
      */
     public void remove(int index)
     {
-        COSDictionary node = get(index + 1, root, 0);
-        remove(node);
+        PageAndPageTreeParent res = get(index + 1, root, 0, null);
+        remove(res.node);
     }
 
     /**
@@ -523,8 +544,12 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
     public void insertBefore(PDPage newPage, PDPage nextPage)
     {
         COSDictionary nextPageDict = nextPage.getCOSObject();
-        COSDictionary parentDict = nextPageDict.getDictionaryObject(COSName.PARENT,
-                COSDictionary.class);
+        COSDictionary parentDict = nextPageDict.getDictionaryObject(COSName.PARENT, COSDictionary.class);
+        if(nextPage.getPageTreeParent() != null)
+        {
+            parentDict = nextPage.getPageTreeParent();
+        }
+        
         COSArray kids = parentDict.getDictionaryObject(COSName.KIDS, COSArray.class);
         boolean found = false;
         for (int i = 0; i < kids.size(); ++i)
@@ -557,6 +582,11 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
         COSDictionary prevPageDict = prevPage.getCOSObject();
         COSDictionary parentDict = prevPageDict.getDictionaryObject(COSName.PARENT,
                 COSDictionary.class);
+        if(prevPage.getPageTreeParent() != null)
+        {
+            parentDict = prevPage.getPageTreeParent();
+        }
+        
         COSArray kids = parentDict.getDictionaryObject(COSName.KIDS, COSArray.class);
         boolean found = false;
         for (int i = 0; i < kids.size(); ++i)
@@ -586,4 +616,19 @@ public class PDPageTree implements COSObjectable, Iterable<PDPage>
             parentDict = (COSDictionary) parentDict.getDictionaryObject(COSName.PARENT);
         } while (parentDict != null);
     }
+
+    /**
+     * A tuple of a page and the parent found when traversing the page tree top down.
+     * NOTE: The page tree parent can be different from page.PARENT (for invalid documents)
+     * hence the need for this class when adding new pages and having to increment kids counters in the page tree
+     */
+    public static class PageAndPageTreeParent {
+        public final COSDictionary node; 
+        public final COSDictionary parent;
+
+        public PageAndPageTreeParent(COSDictionary node, COSDictionary parent) {
+            this.node = node;
+            this.parent = parent;
+        }
+    }
 }
