@@ -57,7 +57,8 @@ public class PDType0Font extends PDFont implements PDVectorFont
     private TrueTypeFont ttf;
 
     /**
-     * Loads a TTF to be embedded into a document as a Type 0 font.
+     * Loads a TTF to be embedded and subset into a document as a Type 0 font. If you are loading a font for AcroForm,
+     * then use the 3-parameter constructor instead.
      *
      * @param doc The PDF document that will hold the embedded font.
      * @param file A TrueType font.
@@ -66,20 +67,25 @@ public class PDType0Font extends PDFont implements PDVectorFont
      */
     public static PDType0Font load(PDDocument doc, File file) throws IOException
     {
-        return new PDType0Font(doc, new TTFParser().parse(file), true, true, false);
+        TrueTypeFont ttf = new TTFParser().parse(file);
+        boolean embedSubset = FontUtils.isSubsettingPermitted(ttf);
+        return new PDType0Font(doc, ttf, embedSubset, true, false);
     }
 
     /**
-     * Loads a TTF to be embedded into a document as a Type 0 font.
+     * Loads a TTF to be embedded and subset into a document as a Type 0 font. If you are loading a font for AcroForm,
+     * then use the 3-parameter constructor instead.
      *
      * @param doc The PDF document that will hold the embedded font.
-     * @param input A TrueType font.
+     * @param input An input stream of a TrueType font. It will be closed before returning.
      * @return A Type0 font with a CIDFontType2 descendant.
      * @throws IOException If there is an error reading the font stream.
      */
     public static PDType0Font load(PDDocument doc, InputStream input) throws IOException
     {
-        return new PDType0Font(doc, new TTFParser().parse(input), true, true, false);
+        TrueTypeFont ttf = new TTFParser().parse(input);
+        boolean embedSubset = FontUtils.isSubsettingPermitted(ttf);
+        return new PDType0Font(doc, ttf, embedSubset, true, false);
     }
 
     /**
@@ -448,7 +454,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
     @Override
     protected float getStandard14Width(int code)
     {
-        throw new UnsupportedOperationException("not suppported");
+        throw new UnsupportedOperationException("not supported");
     }
 
     @Override
@@ -548,6 +554,16 @@ public class PDType0Font extends PDFont implements PDVectorFont
         return descendantFont.isDamaged();
     }
 
+    @Override
+    public boolean isOriginalEmbeddedMissing() {
+        return descendantFont.isOriginalEmbeddedMissing();
+    }
+
+    @Override
+    public boolean isMappingFallbackUsed() {
+        return descendantFont.isMappingFallbackUsed();
+    }
+
     @Override
     public String toString()
     {
@@ -571,4 +587,6 @@ public class PDType0Font extends PDFont implements PDVectorFont
     {
         return descendantFont.hasGlyph(code);
     }
+
+    
 }
