@@ -99,6 +99,8 @@ public class PDType1Font extends PDSimpleFont
     private final FontBoxFont genericFont; // embedded or system font for rendering
     private final boolean isEmbedded;
     private final boolean isDamaged;
+    private boolean isOriginalEmbeddedMissing = false;
+    private boolean isMappingFallbackUsed = false;
     private Matrix fontMatrix;
     private final AffineTransform fontMatrixTransform;
     private BoundingBox fontBBox;
@@ -153,6 +155,7 @@ public class PDType1Font extends PDSimpleFont
                 fontName = "?";
             }
             LOG.warn("Using fallback font " + fontName + " for base font " + getBaseFont());
+            this.isMappingFallbackUsed = true;
         }
         isEmbedded = false;
         isDamaged = false;
@@ -280,6 +283,7 @@ public class PDType1Font extends PDSimpleFont
         }
         else
         {
+            this.isOriginalEmbeddedMissing = true;
             FontMapping<FontBoxFont> mapping = FontMappers.instance().getFontBoxFont(getBaseFont(),
                     fd);
             genericFont = mapping.getFont();
@@ -287,6 +291,7 @@ public class PDType1Font extends PDSimpleFont
             if (mapping.isFallback())
             {
                 LOG.warn("Using fallback font " + genericFont.getName() + " for " + getBaseFont());
+                this.isMappingFallbackUsed = true;
             }
         }
         readEncoding();
@@ -564,12 +569,14 @@ public class PDType1Font extends PDSimpleFont
         {
             return name;
         }
+
         // try alternative name
         String altName = ALT_NAMES.get(name);
         if (altName != null && !name.equals(".notdef") && genericFont.hasGlyph(altName))
         {
             return altName;
         }
+
         // try unicode name
         String unicodes = getGlyphList().toUnicode(name);
         if (unicodes != null && unicodes.length() == 1)
@@ -579,7 +586,25 @@ public class PDType1Font extends PDSimpleFont
             {
                 return uniName;
             }
+            // PDFBOX-4017: no postscript table on Windows 10, and the low uni00NN
+            // names are not found in Symbol font. What works is using the PDF code plus 0xF000
+            // while disregarding encoding from the PDF (because of file from PDFBOX-1606,
+            // makes sense because this segment is about finding the name in a standard font)
+            // TODO bring up better solution than this
+            if ("SymbolMT".equals(genericFont.getName()))
+            {
+                Integer code = SymbolEncoding.INSTANCE.getNameToCodeMap().get(name);
+                if (code != null)
+                {
+                    uniName = getUniNameOfCodePoint(code + 0xF000);
+                    if (genericFont.hasGlyph(uniName))
+                    {
+                        return uniName;
+                    }
+                }
+            }
         }
+
         return ".notdef";
     }
 
@@ -640,4 +665,14 @@ public class PDType1Font extends PDSimpleFont
     {
         return isDamaged;
     }
+
+    @Override
+    public boolean isOriginalEmbeddedMissing() {
+        return isOriginalEmbeddedMissing;
+    }
+
+    @Override
+    public boolean isMappingFallbackUsed() {
+        return isMappingFallbackUsed;
+    }
 }
