@@ -18,6 +18,7 @@
 package org.sejda.sambox.pdmodel.font;
 
 import static java.util.Objects.nonNull;
+import static org.sejda.commons.util.RequireUtils.requireIOCondition;
 
 import java.io.IOException;
 import java.io.InputStream;
@@ -120,16 +121,10 @@ final class Standard14Fonts
         String resourceName = "/org/sejda/sambox/resources/afm/" + afmName + ".afm";
         try (InputStream afmStream = PDType1Font.class.getResourceAsStream(resourceName))
         {
-            if (nonNull(afmStream))
-            {
-                AFMParser parser = new AFMParser(afmStream);
-                FontMetrics metric = parser.parse(true);
-                STANDARD14_AFM_MAP.put(fontName, metric);
-            }
-            else
-            {
-                throw new IOException(resourceName + " not found");
-            }
+            requireIOCondition(nonNull(afmStream), "Unable to load " + resourceName);
+            AFMParser parser = new AFMParser(afmStream);
+            FontMetrics metric = parser.parse(true);
+            STANDARD14_AFM_MAP.put(fontName, metric);
         }
     }
 
