@@ -18,7 +18,7 @@ package org.sejda.sambox.pdmodel.graphics.image;
 
 import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
-import static org.sejda.util.RequireUtils.requireNotNullArg;
+import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 
 import java.awt.Graphics2D;
 import java.awt.Paint;
@@ -35,7 +35,9 @@ import java.util.List;
 
 import javax.imageio.ImageIO;
 
-import org.apache.commons.io.IOUtils;
+import org.sejda.commons.util.IOUtils;
+import org.sejda.io.SeekableSource;
+import org.sejda.io.SeekableSources;
 import org.sejda.sambox.cos.COSArray;
 import org.sejda.sambox.cos.COSBase;
 import org.sejda.sambox.cos.COSName;
@@ -157,18 +159,27 @@ public final class PDImageXObject extends PDXObject implements PDImage
     public static PDImageXObject createFromFile(File file) throws IOException
     {
         requireNotNullArg(file, "Cannot create image from a null file");
+        try (SeekableSource source = SeekableSources.seekableSourceFrom(file))
+        {
+            return createFromSeekableSource(source, file.getName());
+        }
+    }
+
+    public static PDImageXObject createFromSeekableSource(SeekableSource source, String filename)
+            throws IOException
+    {
         // we first try to match the first bytes to some known pattern, so we don't rely on the extension first
-        FileType fileType = FileTypeDetector.detectFileType(file);
+        FileType fileType = FileTypeDetector.detectFileType(source);
 
         if (fileType.equals(FileType.JPEG))
         {
-            return JPEGFactory.createFromFile(file);
+            return JPEGFactory.createFromSeekableSource(source);
         }
         if (fileType.equals(FileType.TIFF))
         {
             try
             {
-                return CCITTFactory.createFromFile(file);
+                return CCITTFactory.createFromSeekableSource(source);
             }
             catch (IOException ex)
             {
@@ -180,16 +191,21 @@ public final class PDImageXObject extends PDXObject implements PDImage
         }
         // last resort, let's see if ImageIO can read it
         BufferedImage image;
-        try {
-            image = ImageIO.read(file);
-        } catch (Exception e) {
-            LOG.warn(String.format("An error occurred while reading image: %s type: %s", file.getName(), fileType), e);
-            throw new UnsupportedImageFormatException(fileType, file.getName(), e);
+        try
+        {
+            image = ImageIO.read(source.asNewInputStream());
+        }
+        catch (Exception e)
+        {
+            LOG.warn(String.format("An error occurred while reading image: %s type: %s", filename,
+                    fileType), e);
+            throw new UnsupportedImageFormatException(fileType, filename, e);
         }
 
-        if(image == null) {
-            LOG.warn(String.format("Could not read image format: %s type: %s", file.getName(), fileType));
-            throw new UnsupportedImageFormatException(fileType, file.getName(), null);
+        if (image == null)
+        {
+            LOG.warn(String.format("Could not read image format: %s type: %s", filename, fileType));
+            throw new UnsupportedImageFormatException(fileType, filename, null);
         }
         return LosslessFactory.createFromImage(image);
     }
@@ -427,6 +443,12 @@ public final class PDImageXObject extends PDXObject implements PDImage
         g.dispose();
         return image2;
     }
+    
+    public boolean hasMask()
+    {
+        COSStream cosStream = getCOSObject().getDictionaryObject(COSName.MASK, COSStream.class);
+        return cosStream != null;
+    }
 
     /**
      * Returns the Mask Image XObject associated with this image, or null if there is none.
@@ -453,6 +475,12 @@ public final class PDImageXObject extends PDXObject implements PDImage
     {
         return getCOSObject().getDictionaryObject(COSName.MASK, COSArray.class);
     }
+    
+    public boolean hasSoftMask()
+    {
+        COSStream cosStream = getCOSObject().getDictionaryObject(COSName.SMASK, COSStream.class);
+        return cosStream != null;
+    }
 
     /**
      * Returns the Soft Mask Image XObject associated with this image, or null if there is none.
@@ -532,6 +560,8 @@ public final class PDImageXObject extends PDXObject implements PDImage
     public void setColorSpace(PDColorSpace cs)
     {
         getCOSObject().setItem(COSName.COLORSPACE, cs != null ? cs.getCOSObject() : null);
+        colorSpace = null;
+        cachedImage = null;
     }
 
     @Override
