@@ -17,10 +17,10 @@
 package org.sejda.sambox.pdmodel.interactive.form;
 
 import static java.util.Arrays.asList;
-import static java.util.Objects.nonNull;
+import static java.util.Objects.isNull;
 import static java.util.Optional.ofNullable;
+import static org.sejda.commons.util.RequireUtils.requireNotNullArg;
 import static org.sejda.io.CountingWritableByteChannel.from;
-import static org.sejda.util.RequireUtils.requireNotNullArg;
 
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Point2D;
@@ -76,7 +76,7 @@ public class AppearanceGeneratorHelper
      *
      * Regardless of other settings in an existing appearance stream Adobe will always use this value.
      */
-    private static final int[] HIGHLIGHT_COLOR = { 153, 193, 215 };
+    private static final float[] HIGHLIGHT_COLOR = { 153 / 255f, 193 / 255f, 215 / 255f };
 
     /**
      * The scaling factor for font units to PDF units
@@ -88,8 +88,6 @@ public class AppearanceGeneratorHelper
      */
     private static final float DEFAULT_FONT_SIZE = 12;
 
-    private static final int MINIMUM_LINES_TO_FIT_IN_A_MULTILINE_FIELD = 5;
-
     /**
      * The default padding applied by Acrobat to the fields bbox.
      */
@@ -105,7 +103,17 @@ public class AppearanceGeneratorHelper
     {
         this.field = field;
         validateAndEnsureAcroFormResources();
-        this.defaultAppearance = field.getDefaultAppearanceString();
+        try
+        {
+            this.defaultAppearance = field.getDefaultAppearanceString();
+        }
+        catch (IOException ex)
+        {
+            throw new IOException(
+                    "Could not process default appearance string '" + field.getDefaultAppearance()
+                            + "' for field '" + field.getFullyQualifiedName() + "'",
+                    ex);
+        }
     }
 
     /*
@@ -166,6 +174,13 @@ public class AppearanceGeneratorHelper
     public void setAppearanceValue(String apValue) throws IOException
     {
         value = apValue;
+        
+        if(field instanceof PDTextField)
+        {
+            // avoid java.lang.IllegalArgumentException: U+00A0 ('nbspace') is not available in this font XYZ
+            // TODO: generic way of handling any character which is not supported by the font?
+            value = value.replaceAll("\\u00A0", " ");
+        }
 
         // Treat multiline field values in single lines as single lime values.
         // This is in line with how Adobe Reader behaves when enetring text
@@ -175,7 +190,7 @@ public class AppearanceGeneratorHelper
         // see PDFBOX-3911
         if (field instanceof PDTextField && !((PDTextField) field).isMultiline())
         {
-            value = apValue.replaceAll(
+            value = value.replaceAll(
                     "\\u000D\\u000A|[\\u000A\\u000B\\u000C\\u000D\\u0085\\u2028\\u2029]", " ");
         }
 
@@ -217,30 +232,13 @@ public class AppearanceGeneratorHelper
                 // TODO support appearances other than "normal"
 
                 PDAppearanceStream appearanceStream;
-                if (nonNull(appearance) && appearance.isStream())
+                if (isValidAppearanceStream(appearance))
                 {
                     appearanceStream = appearance.getAppearanceStream();
                 }
                 else
                 {
-                    appearanceStream = new PDAppearanceStream();
-
-                    // Calculate the entries for the bounding box and the transformation matrix
-                    // settings for the appearance stream
-                    int rotation = resolveRotation(widget);
-                    Matrix matrix = Matrix.getRotateInstance(Math.toRadians(rotation), 0, 0);
-                    Point2D.Float point2D = matrix.transformPoint(rect.getWidth(),
-                            rect.getHeight());
-
-                    PDRectangle bbox = new PDRectangle(Math.abs((float) point2D.getX()),
-                            Math.abs((float) point2D.getY()));
-                    appearanceStream.setBBox(bbox);
-
-                    appearanceStream.setMatrix(calculateMatrix(bbox, rotation));
-                    appearanceStream.setFormType(1);
-
-                    appearanceStream.setResources(new PDResources());
-
+                    appearanceStream = prepareNormalAppearanceStream(widget);
                     appearanceDict.setNormalAppearance(appearanceStream);
                     // TODO support appearances other than "normal"
                 }
@@ -261,10 +259,49 @@ public class AppearanceGeneratorHelper
         }
     }
 
+    private static boolean isValidAppearanceStream(PDAppearanceEntry appearance)
+    {
+        if (isNull(appearance) || !appearance.isStream())
+        {
+            return false;
+        }
+        PDRectangle bbox = appearance.getAppearanceStream().getBBox();
+        if (bbox == null)
+        {
+            return false;
+        }
+        return Math.abs(bbox.getWidth()) > 0 && Math.abs(bbox.getHeight()) > 0;
+    }
+
+    private PDAppearanceStream prepareNormalAppearanceStream(PDAnnotationWidget widget)
+    {
+        PDAppearanceStream appearanceStream = new PDAppearanceStream();
+
+        // Calculate the entries for the bounding box and the transformation matrix
+        // settings for the appearance stream
+        int rotation = resolveRotation(widget);
+        PDRectangle rect = widget.getRectangle();
+        Matrix matrix = Matrix.getRotateInstance(Math.toRadians(rotation), 0, 0);
+        Point2D.Float point2D = matrix.transformPoint(rect.getWidth(), rect.getHeight());
+
+        PDRectangle bbox = new PDRectangle(Math.abs((float) point2D.getX()),
+                Math.abs((float) point2D.getY()));
+        appearanceStream.setBBox(bbox);
+
+        AffineTransform at = calculateMatrix(bbox, rotation);
+        if (!at.isIdentity())
+        {
+            appearanceStream.setMatrix(at);
+        }
+        appearanceStream.setFormType(1);
+        appearanceStream.setResources(new PDResources());
+        return appearanceStream;
+    }
+
     private PDDefaultAppearanceString getWidgetDefaultAppearanceString(PDAnnotationWidget widget)
             throws IOException
     {
-        COSString da = (COSString) widget.getCOSObject().getDictionaryObject(COSName.DA);
+        COSString da = DefaultAppearanceHelper.getDefaultAppearance(widget.getCOSObject().getDictionaryObject(COSName.DA));
         PDResources dr = field.getAcroForm().getDefaultResources();
         try
         {
@@ -461,10 +498,25 @@ public class AppearanceGeneratorHelper
 
         float fontSize = defaultAppearance.getFontSize();
 
-        if (fontSize == 0)
+        // calculate the fontSize (because 0 = autosize)
+        boolean recalculateFontSize = fontSize == 0;
+
+        // always re-calculate the fontSize for text fields
+        // because sometimes the field value changes to a longer string, which won't fit the field anymore when drawn
+        // using the previous font size
+        if (field instanceof PDTextField && !((PDTextField) field).isMultiline())
         {
-            // calculate the fontSize (because 0 = autosize)
-            fontSize = calculateFontSize(font, contentRect);
+            recalculateFontSize = true;
+        }
+
+        if (recalculateFontSize)
+        {
+            float newFontSize = calculateFontSize(font, contentRect);
+            // avoid increasing the font size, if one was already specified
+            if(newFontSize < fontSize || fontSize == 0) 
+            {
+                fontSize = newFontSize;
+            }
         }
 
         // for a listbox generate the highlight rectangle for the selected
@@ -504,6 +556,9 @@ public class AppearanceGeneratorHelper
                 // calculate the position based on the content rectangle
                 y = clipRect.getLowerLeftY() + (clipRect.getHeight() - fontCapAtSize) / 2;
 
+                // shift up slightly, so it does not cover any form field line underneath
+                y += 1;
+
                 // check to ensure that ascents and descents fit
                 if (y - clipRect.getLowerLeftY() < -fontDescentAtSize)
                 {
@@ -690,14 +745,14 @@ public class AppearanceGeneratorHelper
                     paddingEdge.getWidth(), highlightBoxHeight);
             contents.fill();
         }
-        contents.setNonStrokingColor(0);
+        contents.setNonStrokingColor(0f);
     }
 
     private void insertGeneratedListboxAppearance(PDPageContentStream contents,
             PDAppearanceStream appearanceStream, PDRectangle contentRect, PDFont font,
             float fontSize) throws IOException
     {
-        contents.setNonStrokingColor(0);
+        contents.setNonStrokingColor(0f);
 
         int q = field.getQ();
         if (q == PDVariableText.QUADDING_CENTERED || q == PDVariableText.QUADDING_RIGHT)
@@ -739,6 +794,7 @@ public class AppearanceGeneratorHelper
             }
 
             contents.newLineAtOffset(contentRect.getLowerLeftX(), yTextPos);
+            contents.setFont(font, fontSize);
             contents.showText(options.get(i));
 
             if (i != (numOptions - 1))
@@ -790,7 +846,8 @@ public class AppearanceGeneratorHelper
 
             boolean looksLikeFauxMultiline = calculateLineHeight(font,
                     DEFAULT_FONT_SIZE / FONTSCALE) > scaledContentHeight;
-            boolean userTypedMultipleLines = new PlainText(value).getParagraphs().size() > 1;
+            int userTypedLinesCount = new PlainText(value).getParagraphs().size();
+            boolean userTypedMultipleLines = userTypedLinesCount > 1;
 
             if (looksLikeFauxMultiline && !userTypedMultipleLines)
             {
@@ -803,8 +860,9 @@ public class AppearanceGeneratorHelper
             else
             {
                 // calculate a font size which fits at least x lines
+                int minimumLinesToFitInAMultilineField = Math.max(2, userTypedLinesCount);
                 float fontSize = scaledContentHeight
-                        / (MINIMUM_LINES_TO_FIT_IN_A_MULTILINE_FIELD * lineHeight);
+                        / (minimumLinesToFitInAMultilineField * lineHeight);
                 // don't return a font size larger than the default
                 return Math.min(fontSize, DEFAULT_FONT_SIZE);
             }
@@ -847,7 +905,7 @@ public class AppearanceGeneratorHelper
 
     /**
      * Apply padding to a box.
-     * 
+     *
      * @param box box
      * @return the padded box.
      */
