@@ -19,11 +19,11 @@ package org.sejda.sambox.pdmodel.interactive.form;
 import static java.util.Objects.isNull;
 import static java.util.Objects.nonNull;
 import static java.util.Optional.ofNullable;
-import static java.util.function.Function.identity;
-import static java.util.stream.Collectors.toMap;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collection;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -237,19 +237,12 @@ public final class PDAcroForm extends PDDictionaryWrapper
                     annotations.add(annotation);
                 }
                 else if (!annotation.isInvisible() && !annotation.isHidden()
-                        && nonNull(annotation.getNormalAppearanceStream()))
+                        && nonNull(annotation.getNormalAppearanceStream())
+                        && nonNull(annotation.getNormalAppearanceStream().getBBox()))
                 {
-                    if (!isContentStreamWrapped)
-                    {
-                        contentStream = new PDPageContentStream(document, page, AppendMode.APPEND,
-                                true, true);
-                        isContentStreamWrapped = true;
-                    }
-                    else
-                    {
-                        contentStream = new PDPageContentStream(document, page, AppendMode.APPEND,
-                                true);
-                    }
+                    contentStream = new PDPageContentStream(document, page, AppendMode.APPEND, true,
+                            !isContentStreamWrapped);
+                    isContentStreamWrapped = true;
 
                     PDAppearanceStream appearanceStream = annotation.getNormalAppearanceStream();
 
@@ -263,7 +256,7 @@ public final class PDAcroForm extends PDDictionaryWrapper
 
                     // scale the appearance stream - mainly needed for images
                     // in buttons and signatures
-                    boolean needsScaling = resolveNeedsScaling(appearanceStream);
+                    boolean needsScaling = resolveNeedsScaling(annotation, page.getRotation());
 
                     Matrix transformationMatrix = new Matrix();
                     boolean transformed = false;
@@ -275,20 +268,32 @@ public final class PDAcroForm extends PDDictionaryWrapper
                         transformed = true;
                     }
 
+                    // PDFBOX-4693: field could have a rotation matrix
+                    Matrix m = appearanceStream.getMatrix();
+                    int angle = (int) Math
+                            .round(Math.toDegrees(Math.atan2(m.getShearY(), m.getScaleY())));
+                    int rotation = (angle + 360) % 360;
+
                     if (needsScaling)
                     {
                         PDRectangle bbox = appearanceStream.getBBox();
                         PDRectangle fieldRect = annotation.getRectangle();
 
-                        if (bbox.getWidth() - fieldRect.getWidth() != 0
-                                && bbox.getHeight() - fieldRect.getHeight() != 0)
+                        float xScale;
+                        float yScale;
+                        if (rotation == 90 || rotation == 270)
                         {
-                            float xScale = fieldRect.getWidth() / bbox.getWidth();
-                            float yScale = fieldRect.getHeight() / bbox.getHeight();
-                            Matrix scalingMatrix = Matrix.getScaleInstance(xScale, yScale);
-                            transformationMatrix.concatenate(scalingMatrix);
-                            transformed = true;
+                            xScale = fieldRect.getWidth() / bbox.getHeight();
+                            yScale = fieldRect.getHeight() / bbox.getWidth();
                         }
+                        else
+                        {
+                            xScale = fieldRect.getWidth() / bbox.getWidth();
+                            yScale = fieldRect.getHeight() / bbox.getHeight();
+                        }
+                        Matrix scalingMatrix = Matrix.getScaleInstance(xScale, yScale);
+                        transformationMatrix.concatenate(scalingMatrix);
+                        transformed = true;
                     }
 
                     if (transformed)
@@ -379,7 +384,7 @@ public final class PDAcroForm extends PDDictionaryWrapper
      * 
      * @param fields
      */
-    public void addFields(List<PDField> toAdd)
+    public void addFields(Collection<PDField> toAdd)
     {
         COSArray fields = ofNullable(
                 getCOSObject().getDictionaryObject(COSName.FIELDS, COSArray.class))
@@ -681,18 +686,38 @@ public final class PDAcroForm extends PDDictionaryWrapper
      * @param appearanceStream
      * @return the need for a scaling transformation.
      */
-    private boolean resolveNeedsScaling(PDAppearanceStream appearanceStream)
+    private boolean resolveNeedsScaling(PDAnnotation annotation, int rotation)
     {
+        PDAppearanceStream appearanceStream = annotation.getNormalAppearanceStream();
         // Check if there is a transformation within the XObjects content
         PDResources resources = appearanceStream.getResources();
-        return resources != null && resources.getXObjectNames().iterator().hasNext();
+        if (resources != null && resources.getXObjectNames().iterator().hasNext())
+        {
+            return true;
+        }
+        PDRectangle bbox = appearanceStream.getBBox();
+        PDRectangle fieldRect = annotation.getRectangle();
+        if (rotation == 90 || rotation == 270)
+        {
+            return Float.compare(bbox.getWidth(), fieldRect.getHeight()) != 0
+                    || Float.compare(bbox.getHeight(), fieldRect.getWidth()) != 0;
+        }
+        return Float.compare(bbox.getWidth(), fieldRect.getWidth()) != 0
+                || Float.compare(bbox.getHeight(), fieldRect.getHeight()) != 0;
     }
 
     private Map<COSDictionary, PDAnnotationWidget> widgets(List<PDField> fields)
     {
-        return fields.stream().flatMap(f -> f.getWidgets().stream())
-                .collect(toMap(w -> w.getCOSObject(), identity()));
+        Map<COSDictionary, PDAnnotationWidget> widgetMap = new HashMap<>();
+        for (PDField field : fields)
+        {
+            for (PDAnnotationWidget widget : field.getWidgets())
+            {
+                widgetMap.put(widget.getCOSObject(), widget);
+            }
+        }
 
+        return widgetMap;
     }
 
     private void removeFields(List<PDField> fields)
