@@ -23,6 +23,8 @@ import java.util.HashMap;
 import java.util.Map;
 
 import org.sejda.sambox.pdmodel.font.PDFont;
+import org.sejda.sambox.pdmodel.graphics.color.PDColor;
+import org.sejda.sambox.pdmodel.graphics.state.RenderingMode;
 import org.sejda.sambox.util.Matrix;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -59,12 +61,22 @@ public class TextPosition
     private final PDFont font;
     private final float fontSize;
     private final int fontSizePt;
+    
+    private final PDColor color;
+    private final RenderingMode renderingMode;
 
     // mutable
     private float[] widths;
     private String unicode;
     private float direction = -1;
 
+    public TextPosition(int pageRotation, float pageWidth, float pageHeight, Matrix textMatrix,
+                        float endX, float endY, float maxHeight, float individualWidth, float spaceWidth,
+                        String unicode, int[] charCodes, PDFont font, float fontSize, int fontSizeInPt) {
+        this(pageRotation, pageWidth, pageHeight, textMatrix, endX, endY, maxHeight, individualWidth, spaceWidth, 
+                unicode, charCodes, font, fontSize, fontSizeInPt, null, null);
+    }
+
     /**
      * Constructor.
      *
@@ -84,8 +96,9 @@ public class TextPosition
      * @param fontSizeInPt The font size in pt units (see {@link #getFontSizeInPt()} for details).
      */
     public TextPosition(int pageRotation, float pageWidth, float pageHeight, Matrix textMatrix,
-            float endX, float endY, float maxHeight, float individualWidth, float spaceWidth,
-            String unicode, int[] charCodes, PDFont font, float fontSize, int fontSizeInPt)
+                        float endX, float endY, float maxHeight, float individualWidth, float spaceWidth,
+                        String unicode, int[] charCodes, PDFont font, float fontSize, int fontSizeInPt,
+                        PDColor color, RenderingMode renderingMode)
     {
         this.textMatrix = textMatrix;
 
@@ -106,6 +119,8 @@ public class TextPosition
         this.font = font;
         this.fontSize = fontSize;
         this.fontSizePt = fontSizeInPt;
+        this.color = color;
+        this.renderingMode = renderingMode;
 
         x = getXRot(rotationAngle);
         if (rotationAngle == 0 || rotationAngle == 180)
@@ -195,9 +210,17 @@ public class TextPosition
     }
 
     /**
-     * Return the direction/orientation of the string in this object based on its text matrix.
+     * Return the direction/orientation of the string in this object based on its text matrix. Only angles of 0, 90,
+     * 180, or 270 are supported. To get other angles, use this code:
      * 
-     * @return The direction of the text (0, 90, 180, or 270)
+     * <pre>
+     * TextPosition text = ...
+     * Matrix m = text.getTextMatrix().clone();
+     * m.concatenate(text.getFont().getFontMatrix());
+     * int angle = (int) Math.round(Math.toDegrees(Math.atan2(m.getShearY(), m.getScaleY())));
+     * </pre>
+     *
+     * @return The direction of the text (0, 90, 180, or 270).
      */
     public float getDir()
     {
@@ -270,7 +293,9 @@ public class TextPosition
 
     /**
      * This will get the page rotation adjusted x position of the character. This is adjusted based on page rotation so
-     * that the upper left is 0,0.
+     * that the upper left is 0,0 which is unlike PDF coordinates, which start at the bottom left. See also
+     * <a href="https://stackoverflow.com/questions/57067372/">this answer by Michael Klink</a> for further details and
+     * <a href="https://issues.apache.org/jira/browse/PDFBOX-4597">PDFBOX-4597</a> for a sample file.
      *
      * @return The x coordinate of the character.
      */
@@ -281,8 +306,12 @@ public class TextPosition
 
     /**
      * This will get the text direction adjusted x position of the character. This is adjusted based on text direction
-     * so that the first character in that direction is in the upper left at 0,0.
-     *
+     * so that the first character in that direction is in the upper left at 0,0. This method ignores the page rotation
+     * but takes the text rotation (see {@link #getDir() getDir()}) and adjusts the coordinates to awt. This is useful
+     * when doing text extraction, to compare the glyph positions when imagining these to be horizontal. See also
+     * <a href="https://stackoverflow.com/questions/57067372/">this answer by Michael Klink</a> for further details and
+     * <a href="https://issues.apache.org/jira/browse/PDFBOX-4597">PDFBOX-4597</a> for a sample file.
+     * 
      * @return The x coordinate of the text.
      */
     public float getXDirAdj()
@@ -320,8 +349,11 @@ public class TextPosition
 
     /**
      * This will get the y position of the text, adjusted so that 0,0 is upper left and it is adjusted based on the page
-     * rotation.
-     *
+     * rotation. This method ignores the page rotation but takes the text rotation and adjusts the coordinates to awt.
+     * This is useful when doing text extraction, to compare the glyph positions when imagining these to be horizontal.
+     * See also <a href="https://stackoverflow.com/questions/57067372/">this answer by Michael Klink</a> for further
+     * details and <a href="https://issues.apache.org/jira/browse/PDFBOX-4597">PDFBOX-4597</a> for a sample file.
+     * 
      * @return The adjusted y coordinate of the character.
      */
     public float getY()
@@ -590,20 +622,20 @@ public class TextPosition
             }
             // diacritic completely covers this character and therefore we assume that this is the
             // character the diacritic belongs to
-            else if (diacXStart < currCharXStart && diacXEnd > currCharXEnd)
+            else if (diacXStart < currCharXStart)
             {
                 insertDiacritic(i, diacritic);
                 wasAdded = true;
             }
             // otherwise, The diacritic modifies this character because its completely
             // contained by the character width
-            else if (diacXStart >= currCharXStart && diacXEnd <= currCharXEnd)
+            else if (diacXEnd <= currCharXEnd)
             {
                 insertDiacritic(i, diacritic);
                 wasAdded = true;
             }
             // last character in the TextPosition so we add diacritic to the end
-            else if (diacXStart >= currCharXStart && diacXEnd > currCharXEnd && i == strLen - 1)
+            else if (i == strLen - 1)
             {
                 insertDiacritic(i, diacritic);
                 wasAdded = true;
@@ -693,7 +725,8 @@ public class TextPosition
     public boolean isVisible()
     {
         Rectangle2D.Float rectangle = new Rectangle2D.Float(0, 0, pageWidth, pageHeight);
-        if(this.rotation == 90 || this.rotation == 270) {
+        if (this.rotation == 90 || this.rotation == 270)
+        {
             // flip width and height
             rectangle = new Rectangle2D.Float(0, 0, pageHeight, pageWidth);
         }
@@ -765,6 +798,14 @@ public class TextPosition
         return pageWidth;
     }
 
+    public PDColor getColor() {
+        return color;
+    }
+
+    public RenderingMode getRenderingMode() {
+        return renderingMode;
+    }
+
     @Override
     public boolean equals(Object o)
     {
@@ -823,10 +864,6 @@ public class TextPosition
         {
             return false;
         }
-        if (Float.compare(that.direction, direction) != 0)
-        {
-            return false;
-        }
         if (textMatrix != null ? !textMatrix.equals(that.textMatrix) : that.textMatrix != null)
         {
             return false;
@@ -835,15 +872,17 @@ public class TextPosition
         {
             return false;
         }
-        if (font != null ? !font.equals(that.font) : that.font != null)
+        if (renderingMode != null ? !renderingMode.equals(that.renderingMode) : that.renderingMode != null)
         {
             return false;
         }
-        if (!Arrays.equals(widths, that.widths))
+        if (color != null ? !color.equals(that.color) : that.color != null)
         {
             return false;
         }
-        return unicode != null ? unicode.equals(that.unicode) : that.unicode == null;
+        return font != null ? font.equals(that.font) : that.font == null;
+
+        // If changing this method, do not compare mutable fields (PDFBOX-4701)
 
     }
 
@@ -864,6 +903,8 @@ public class TextPosition
         result = 31 * result + (font != null ? font.hashCode() : 0);
         result = 31 * result + Float.floatToIntBits(fontSize);
         result = 31 * result + fontSizePt;
+        result = 31 * result + (color != null ? color.hashCode() : 0);
+        result = 31 * result + (renderingMode != null ? renderingMode.hashCode() : 0);
         return result;
     }
 }
